@@ -1,14 +1,14 @@
 #!/bin/sh
-# Copyright 2019, Collabora, Ltd.
+# Copyright 2019-2024, Collabora, Ltd.
 # SPDX-License-Identifier: BSL-1.0
-# Author: Ryan Pavlik <ryan.pavlik@collabora.com>
+# Author: Rylie Pavlik <rylie.pavlik@collabora.com>
 
 # Formats all the source files in this project
 
 set -e
 
 if [ ! "${CLANGFORMAT}" ]; then
-        for fn in clang-format-11 clang-format-10 clang-format-9 clang-format-8 clang-format-7 clang-format-6.0 clang-format; do
+        for fn in clang-format-14 clang-format-13 clang-format-12 clang-format-11 clang-format-10 clang-format-9 clang-format-8 clang-format-7 clang-format-6.0 clang-format; do
                 if command -v $fn > /dev/null; then
                         CLANGFORMAT=$fn
                         break
@@ -22,7 +22,10 @@ if [ ! "${CLANGFORMAT}" ]; then
 fi
 
 (
-        cd $(dirname $0)/..
+        ${CLANGFORMAT} --version
+
+        cd "$(dirname "$0")/.."
+
         find \
                 src/xrt/auxiliary \
                 src/xrt/compositor \
@@ -31,7 +34,9 @@ fi
                 src/xrt/ipc \
                 src/xrt/state_trackers \
                 src/xrt/targets \
+                src/xrt/tracking \
                 tests \
                 \( -name "*.c" -o -name "*.cpp" -o -name "*.h" -o -name "*.hpp" \) \
-                -exec ${CLANGFORMAT} -i -style=file \{\} +
+                -and -not \( -ipath \*/.cxx/\* \) \
+                -exec "${CLANGFORMAT}" -i -style=file \{\} +
 )
