@@ -1,6 +1,6 @@
-// Copyright 2020, Collabora, Ltd.
+// Copyright 2020-2021, Collabora, Ltd.
 // SPDX-License-Identifier: BSL-1.0
-// Author: Ryan Pavlik <ryan.pavlik@collabora.com>
+// Author: Rylie Pavlik <rylie.pavlik@collabora.com>
 
 #include "android.app.h"
 
@@ -11,8 +11,7 @@ Service::Meta::Meta() : MetaBaseDroppable(Service::getTypeName()) {
 }
 Activity::Meta::Meta()
     : MetaBaseDroppable(Activity::getTypeName()),
-      getSystemService(classRef().getMethod(
-          "getSystemService", "(Ljava/lang/String;)Ljava/lang/Object;")),
+      getWindow(classRef().getMethod("getWindow", "()Landroid/view/Window;")),
       setVrModeEnabled(classRef().getMethod(
           "setVrModeEnabled", "(ZLandroid/content/ComponentName;)V")) {
     MetaBaseDroppable::dropClassRef();
