@@ -1,6 +1,6 @@
-// Copyright 2020, Collabora, Ltd.
+// Copyright 2020-2021, Collabora, Ltd.
 // SPDX-License-Identifier: BSL-1.0
-// Author: Ryan Pavlik <ryan.pavlik@collabora.com>
+// Author: Rylie Pavlik <rylie.pavlik@collabora.com>
 
 #pragma once
 
@@ -51,7 +51,7 @@ class Point : public ObjectWrapperBase {
          * Singleton accessor
          */
         static Meta &data() {
-            static Meta instance;
+            static Meta instance{};
             return instance;
         }
 
@@ -59,6 +59,34 @@ class Point : public ObjectWrapperBase {
         Meta();
     };
 };
+
+class PixelFormat : public ObjectWrapperBase {
+public:
+  using ObjectWrapperBase::ObjectWrapperBase;
+  static constexpr const char *getTypeName() noexcept {
+      return "android/graphics/PixelFormat";
+  }
+
+  static int32_t OPAQUE();
+
+  /*!
+   * Class metadata
+   */
+  struct Meta : public MetaBaseDroppable {
+      impl::StaticFieldId<int32_t> OPAQUE;
+
+      /*!
+       * Singleton accessor
+       */
+      static Meta &data() {
+          static Meta instance{};
+          return instance;
+      }
+
+    private:
+        Meta();
+    };
+};
 } // namespace android::graphics
 } // namespace wrap
 #include "android.graphics.impl.h"
