@@ -1,12 +1,19 @@
-// Copyright 2020, Collabora, Ltd.
+// Copyright 2020-2023, Collabora, Ltd.
 // SPDX-License-Identifier: BSL-1.0
-// Author: Ryan Pavlik <ryan.pavlik@collabora.com>
+// Author: Rylie Pavlik <rylie.pavlik@collabora.com>
 
 #pragma once
 
 #include "ObjectWrapperBase.h"
 #include <string>
 
+namespace wrap {
+namespace android::content {
+class Context;
+} // namespace android::content
+
+} // namespace wrap
+
 namespace wrap {
 namespace android::provider {
 /*!
@@ -30,17 +37,29 @@ class Settings : public ObjectWrapperBase {
      */
     static std::string ACTION_VR_LISTENER_SETTINGS();
 
+    /*!
+     * Wrapper for the canDrawOverlays static method
+     *
+     * Java prototype:
+     * `public static boolean canDrawOverlays(android.content.Context);`
+     *
+     * JNI signature: (Landroid/content/Context;)Z
+     *
+     */
+    static bool canDrawOverlays(const content::Context &context);
+
     /*!
      * Class metadata
      */
     struct Meta : public MetaBase {
         impl::StaticFieldId<std::string> ACTION_VR_LISTENER_SETTINGS;
+        jni::method_t canDrawOverlays;
 
         /*!
          * Singleton accessor
          */
         static Meta &data() {
-            static Meta instance;
+            static Meta instance{};
             return instance;
         }
 
@@ -48,6 +67,7 @@ class Settings : public ObjectWrapperBase {
         Meta();
     };
 };
+
 } // namespace android::provider
 } // namespace wrap
 #include "android.provider.impl.h"
