@@ -22,6 +22,11 @@ void igPlotTimings(const char *label,
 
 void igToggleButton(const char *str_id, bool *v);
 
+void igImageBg(ImTextureID user_texture_id,
+               const ImVec2 size,
+               const ImVec2 uv0, const ImVec2 uv1,
+               const ImVec4 tint_col, const ImVec4 border_col, const ImVec4 bg_col);
+
 #ifdef __cplusplus
 }
 #endif
