@@ -40,7 +40,10 @@ struct m_imu_3dof
 		uint64_t timestamp_ns;
 		struct xrt_vec3 gyro;  //!< Angular velocity
 		struct xrt_vec3 accel; //!< Acceleration
-		float delta_ms;
+		double delta_ms;
+		float accel_length;
+		float gyro_length;
+		float gyro_biased_length;
 	} last;
 
 	enum m_imu_3dof_state state;
@@ -57,12 +60,24 @@ struct m_imu_3dof
 		uint64_t level_timestamp_ns;
 		struct xrt_vec3 error_axis;
 		float error_angle;
+		bool is_accel;
+		bool is_rotating;
 	} grav;
+
+	// gyro bias correction
+	struct
+	{
+		struct xrt_vec3 value;
+		bool manually_fire;
+	} gyro_bias;
 };
 
 void
 m_imu_3dof_init(struct m_imu_3dof *f, int flags);
 
+void
+m_imu_3dof_reset(struct m_imu_3dof *f);
+
 void
 m_imu_3dof_close(struct m_imu_3dof *f);
 
@@ -71,7 +86,7 @@ m_imu_3dof_add_vars(struct m_imu_3dof *f, void *root, const char *prefix);
 
 void
 m_imu_3dof_update(struct m_imu_3dof *f,
-                  uint64_t timepoint_ns,
+                  uint64_t timestamp_ns,
                   const struct xrt_vec3 *accel,
                   const struct xrt_vec3 *gyro);
 
