@@ -3,10 +3,10 @@
 /*!
  * @file
  * @brief  EGL API wrapper header.
- * @author Ryan Pavlik <ryan.pavlik@collabora.com>
+ * @author Rylie Pavlik <rylie.pavlik@collabora.com>
  * @ingroup aux_ogl
  */
 
 #pragma once
 
-#include "glad/egl.h"
+#include "glad/egl.h" // IWYU pragma: export
