@@ -9,4 +9,4 @@
 
 #pragma once
 
-#include "glad/gl.h"
+#include "glad/gl.h" // IWYU pragma: export
