@@ -1,9 +1,10 @@
-// Copyright 2019-2020, Collabora, Ltd.
+// Copyright 2019-2022, Collabora, Ltd.
 // SPDX-License-Identifier: BSL-1.0
 /*!
  * @file
  * @brief  @ref xrt_frame_sink converters and other helpers.
  * @author Jakob Bornecrantz <jakob@collabora.com>
+ * @author Moshi Turner <moshiturner@protonmail.com>
  * @ingroup aux_util
  */
 
@@ -13,6 +14,7 @@
 #include "util/u_sink.h"
 #include "util/u_frame.h"
 #include "util/u_format.h"
+#include "util/u_trace_marker.h"
 
 #include <stdio.h>
 
@@ -38,12 +40,35 @@ struct u_sink_converter
 	struct xrt_frame_node node;
 
 	struct xrt_frame_sink *downstream;
-	struct xrt_frame_sink *downstream2;
 
 	enum xrt_format format;
 };
 
 
+/*
+ *
+ * L8 functions.
+ *
+ */
+
+static void
+from_L8_to_R8G8B8(struct xrt_frame *dst_frame, uint32_t w, uint32_t h, size_t stride, const uint8_t *data)
+{
+	SINK_TRACE_MARKER();
+
+	for (uint32_t y = 0; y < h; y++) {
+		for (uint32_t x = 0; x < w; x += 1) {
+			const uint8_t *src = data;
+			uint8_t *dst = dst_frame->data;
+
+			src = src + (y * stride) + x;
+			dst = dst + (y * dst_frame->stride) + (x * 3);
+			dst[2] = dst[1] = dst[0] = src[0];
+		}
+	}
+}
+
+
 /*
  *
  * YUV functions.
@@ -192,6 +217,8 @@ YUV444_to_R8G8B8(const uint8_t *input, uint8_t *dst)
 static void
 from_YUYV422_to_R8G8B8(struct xrt_frame *dst_frame, uint32_t w, uint32_t h, size_t stride, const uint8_t *data)
 {
+	SINK_TRACE_MARKER();
+
 	for (uint32_t y = 0; y < h; y++) {
 		for (uint32_t x = 0; x < w; x += 2) {
 			const uint8_t *src = data;
@@ -204,9 +231,26 @@ from_YUYV422_to_R8G8B8(struct xrt_frame *dst_frame, uint32_t w, uint32_t h, size
 	}
 }
 
+static void
+from_YUYV422_to_L8(struct xrt_frame *dst_frame, uint32_t w, uint32_t h, size_t stride, const uint8_t *data)
+{
+	SINK_TRACE_MARKER();
+
+	for (uint32_t y = 0; y < h; y++) {
+		const uint8_t *src = data + (y * stride);
+		uint8_t *dst = dst_frame->data + (y * dst_frame->stride);
+
+		for (uint32_t x = 0; x < w; x++) {
+			dst[x] = src[x * 2];
+		}
+	}
+}
+
 static void
 from_UYVY422_to_R8G8B8(struct xrt_frame *dst_frame, uint32_t w, uint32_t h, size_t stride, const uint8_t *data)
 {
+	SINK_TRACE_MARKER();
+
 	for (uint32_t y = 0; y < h; y++) {
 		for (uint32_t x = 0; x < w; x += 2) {
 			const uint8_t *src = data;
@@ -223,6 +267,8 @@ from_UYVY422_to_R8G8B8(struct xrt_frame *dst_frame, uint32_t w, uint32_t h, size
 static void
 from_YUV888_to_R8G8B8(struct xrt_frame *dst_frame, uint32_t w, uint32_t h, size_t stride, const uint8_t *data)
 {
+	SINK_TRACE_MARKER();
+
 	for (uint32_t y = 0; y < h; y++) {
 		for (uint32_t x = 0; x < w; x++) {
 			const uint8_t *src = data;
@@ -262,6 +308,8 @@ check_header(size_t size, const uint8_t *data)
 static bool
 from_MJPEG_to_R8G8B8(struct xrt_frame *dst_frame, size_t size, const uint8_t *data)
 {
+	SINK_TRACE_MARKER();
+
 	if (!check_header(size, data)) {
 		return false;
 	}
@@ -302,6 +350,8 @@ from_MJPEG_to_R8G8B8(struct xrt_frame *dst_frame, size_t size, const uint8_t *da
 static bool
 from_MJPEG_to_YUV888(struct xrt_frame *dst_frame, size_t size, const uint8_t *data)
 {
+	SINK_TRACE_MARKER();
+
 	if (!check_header(size, data)) {
 		return false;
 	}
@@ -350,11 +400,13 @@ from_MJPEG_to_YUV888(struct xrt_frame *dst_frame, size_t size, const uint8_t *da
 static void
 from_BAYER_GR8_to_R8G8B8(struct xrt_frame *dst_frame, uint32_t w, uint32_t h, size_t stride, const uint8_t *data)
 {
+	SINK_TRACE_MARKER();
+
 	const uint8_t *src_data = data;
-	uint32_t src_stride = stride;
+	size_t src_stride = stride;
 
 	uint8_t *dst_data = dst_frame->data;
-	uint32_t dst_stride = dst_frame->stride;
+	size_t dst_stride = dst_frame->stride;
 
 	for (uint32_t y = 0; y < h; y++) {
 		const uint8_t *src0 = src_data + (y * 2) * src_stride;
@@ -424,8 +476,36 @@ create_frame_with_format(struct xrt_frame *xf, enum xrt_format format, struct xr
 }
 
 static void
-receive_frame_r8g8b8_or_l8(struct xrt_frame_sink *xs, struct xrt_frame *xf)
+convert_frame_l8(struct xrt_frame_sink *xs, struct xrt_frame *xf)
 {
+	SINK_TRACE_MARKER();
+
+	struct u_sink_converter *s = (struct u_sink_converter *)xs;
+
+	struct xrt_frame *converted = NULL;
+
+	switch (xf->format) {
+	case XRT_FORMAT_L8: s->downstream->push_frame(s->downstream, xf); return;
+	case XRT_FORMAT_YUYV422:
+		if (!create_frame_with_format(xf, XRT_FORMAT_L8, &converted)) {
+			return;
+		}
+		from_YUYV422_to_L8(converted, xf->width, xf->height, xf->stride, xf->data);
+		break;
+	default: U_LOG_E("Cannot convert from '%s' to L8!", u_format_str(xf->format)); return;
+	}
+
+	s->downstream->push_frame(s->downstream, converted);
+
+	// Refcount in case it's being held downstream.
+	xrt_frame_reference(&converted, NULL);
+}
+
+static void
+convert_frame_r8g8b8_or_l8(struct xrt_frame_sink *xs, struct xrt_frame *xf)
+{
+	SINK_TRACE_MARKER();
+
 	struct u_sink_converter *s = (struct u_sink_converter *)xs;
 
 	struct xrt_frame *converted = NULL;
@@ -465,11 +545,13 @@ receive_frame_r8g8b8_or_l8(struct xrt_frame_sink *xs, struct xrt_frame *xf)
 			return;
 		}
 		if (!from_MJPEG_to_R8G8B8(converted, xf->size, xf->data)) {
+			// Make sure to free frame when we fail to decode.
+			xrt_frame_reference(&converted, NULL);
 			return;
 		}
 		break;
 #endif
-	default: U_LOG_E("Can not convert from '%s' to R8G8B8 or L8!", u_format_str(xf->format)); return;
+	default: U_LOG_E("Cannot convert from '%s' to R8G8B8 or L8!", u_format_str(xf->format)); return;
 	}
 
 	s->downstream->push_frame(s->downstream, converted);
@@ -479,8 +561,26 @@ receive_frame_r8g8b8_or_l8(struct xrt_frame_sink *xs, struct xrt_frame *xf)
 }
 
 static void
-receive_frame_r8g8b8_bayer_or_l8(struct xrt_frame_sink *xs, struct xrt_frame *xf)
+convert_frame_r8g8b8_r8g8b8a8_r8g8b8x8_or_l8(struct xrt_frame_sink *xs, struct xrt_frame *xf)
 {
+	SINK_TRACE_MARKER();
+
+	struct u_sink_converter *s = (struct u_sink_converter *)xs;
+
+	switch (xf->format) {
+	case XRT_FORMAT_L8:
+	case XRT_FORMAT_R8G8B8A8:
+	case XRT_FORMAT_R8G8B8X8:
+	case XRT_FORMAT_R8G8B8: s->downstream->push_frame(s->downstream, xf); return;
+	default: convert_frame_r8g8b8_or_l8(xs, xf);
+	}
+}
+
+static void
+convert_frame_r8g8b8_bayer_or_l8(struct xrt_frame_sink *xs, struct xrt_frame *xf)
+{
+	SINK_TRACE_MARKER();
+
 	struct u_sink_converter *s = (struct u_sink_converter *)xs;
 
 	struct xrt_frame *converted = NULL;
@@ -513,11 +613,13 @@ receive_frame_r8g8b8_bayer_or_l8(struct xrt_frame_sink *xs, struct xrt_frame *xf
 			return;
 		}
 		if (!from_MJPEG_to_R8G8B8(converted, xf->size, xf->data)) {
+			// Make sure to free frame when we fail to decode.
+			xrt_frame_reference(&converted, NULL);
 			return;
 		}
 		break;
 #endif
-	default: U_LOG_E("Can not convert from '%s' to R8G8B8 or L8!", u_format_str(xf->format)); return;
+	default: U_LOG_E("Cannot convert from '%s' to R8G8B8 or L8!", u_format_str(xf->format)); return;
 	}
 
 	s->downstream->push_frame(s->downstream, converted);
@@ -527,14 +629,22 @@ receive_frame_r8g8b8_bayer_or_l8(struct xrt_frame_sink *xs, struct xrt_frame *xf
 }
 
 static void
-receive_frame_r8g8b8(struct xrt_frame_sink *xs, struct xrt_frame *xf)
+convert_frame_r8g8b8(struct xrt_frame_sink *xs, struct xrt_frame *xf)
 {
+	SINK_TRACE_MARKER();
+
 	struct u_sink_converter *s = (struct u_sink_converter *)xs;
 
 	struct xrt_frame *converted = NULL;
 
 	switch (xf->format) {
 	case XRT_FORMAT_R8G8B8: s->downstream->push_frame(s->downstream, xf); return;
+	case XRT_FORMAT_L8:
+		if (!create_frame_with_format(xf, XRT_FORMAT_R8G8B8, &converted)) {
+			return;
+		}
+		from_L8_to_R8G8B8(converted, xf->width, xf->height, xf->stride, xf->data);
+		break;
 	case XRT_FORMAT_BAYER_GR8:;
 		uint32_t w = xf->width / 2;
 		uint32_t h = xf->height / 2;
@@ -567,11 +677,13 @@ receive_frame_r8g8b8(struct xrt_frame_sink *xs, struct xrt_frame *xf)
 			return;
 		}
 		if (!from_MJPEG_to_R8G8B8(converted, xf->size, xf->data)) {
+			// Make sure to free frame when we fail to decode.
+			xrt_frame_reference(&converted, NULL);
 			return;
 		}
 		break;
 #endif
-	default: U_LOG_E("Can not convert from '%s' to R8G8B8!", u_format_str(xf->format)); return;
+	default: U_LOG_E("Cannot convert from '%s' to R8G8B8!", u_format_str(xf->format)); return;
 	}
 
 	s->downstream->push_frame(s->downstream, converted);
@@ -581,8 +693,46 @@ receive_frame_r8g8b8(struct xrt_frame_sink *xs, struct xrt_frame *xf)
 }
 
 static void
-receive_frame_yuv_yuyv_uyvy_or_l8(struct xrt_frame_sink *xs, struct xrt_frame *xf)
+convert_frame_rgb_yuv_yuyv_uyvy_or_l8(struct xrt_frame_sink *xs, struct xrt_frame *xf)
 {
+	SINK_TRACE_MARKER();
+
+	struct u_sink_converter *s = (struct u_sink_converter *)xs;
+
+	struct xrt_frame *converted = NULL;
+
+	switch (xf->format) {
+	case XRT_FORMAT_R8G8B8:
+	case XRT_FORMAT_L8:
+	case XRT_FORMAT_YUYV422:
+	case XRT_FORMAT_UYVY422:
+	case XRT_FORMAT_YUV888: s->downstream->push_frame(s->downstream, xf); return;
+#ifdef XRT_HAVE_JPEG
+	case XRT_FORMAT_MJPEG:
+		if (!create_frame_with_format(xf, XRT_FORMAT_YUV888, &converted)) {
+			return;
+		}
+		if (!from_MJPEG_to_YUV888(converted, xf->size, xf->data)) {
+			return;
+		}
+		break;
+#endif
+	default:
+		U_LOG_E("Cannot convert from '%s' to either R8G8B8, YUV, YUYV, UYVY or L8!", u_format_str(xf->format));
+		return;
+	}
+
+	s->downstream->push_frame(s->downstream, converted);
+
+	// Refcount in case it's being held downstream.
+	xrt_frame_reference(&converted, NULL);
+}
+
+static void
+convert_frame_yuv_yuyv_uyvy_or_l8(struct xrt_frame_sink *xs, struct xrt_frame *xf)
+{
+	SINK_TRACE_MARKER();
+
 	struct u_sink_converter *s = (struct u_sink_converter *)xs;
 
 	struct xrt_frame *converted = NULL;
@@ -604,7 +754,7 @@ receive_frame_yuv_yuyv_uyvy_or_l8(struct xrt_frame_sink *xs, struct xrt_frame *x
 #endif
 	default:
 		U_LOG_E(
-		    "Can not convert from '%s' to either YUV, YUYV, UYVY or "
+		    "Cannot convert from '%s' to either YUV, YUYV, UYVY or "
 		    "L8!",
 		    u_format_str(xf->format));
 		return;
@@ -617,8 +767,10 @@ receive_frame_yuv_yuyv_uyvy_or_l8(struct xrt_frame_sink *xs, struct xrt_frame *x
 }
 
 static void
-receive_frame_yuv_or_yuyv(struct xrt_frame_sink *xs, struct xrt_frame *xf)
+convert_frame_yuv_or_yuyv(struct xrt_frame_sink *xs, struct xrt_frame *xf)
 {
+	SINK_TRACE_MARKER();
+
 	struct u_sink_converter *s = (struct u_sink_converter *)xs;
 
 	struct xrt_frame *converted = NULL;
@@ -636,7 +788,7 @@ receive_frame_yuv_or_yuyv(struct xrt_frame_sink *xs, struct xrt_frame *xf)
 		}
 		break;
 #endif
-	default: U_LOG_E("Can not convert from '%s' to either YUV or YUYV!", u_format_str(xf->format)); return;
+	default: U_LOG_E("Cannot convert from '%s' to either YUV or YUYV!", u_format_str(xf->format)); return;
 	}
 
 	s->downstream->push_frame(s->downstream, converted);
@@ -646,8 +798,10 @@ receive_frame_yuv_or_yuyv(struct xrt_frame_sink *xs, struct xrt_frame *xf)
 }
 
 XRT_MAYBE_UNUSED static void
-receive_frame_bayer(struct xrt_frame_sink *xs, struct xrt_frame *xf)
+convert_frame_bayer(struct xrt_frame_sink *xs, struct xrt_frame *xf)
 {
+	SINK_TRACE_MARKER();
+
 	struct u_sink_converter *s = (struct u_sink_converter *)xs;
 
 	uint32_t w = xf->width / 2;
@@ -687,13 +841,18 @@ destroy(struct xrt_frame_node *node)
 
 void
 u_sink_create_format_converter(struct xrt_frame_context *xfctx,
-                               enum xrt_format f,
+                               enum xrt_format format,
                                struct xrt_frame_sink *downstream,
                                struct xrt_frame_sink **out_xfs)
 {
-	if (f != XRT_FORMAT_R8G8B8) {
-		U_LOG_E("Format '%s' not supported", u_format_str(f));
-		return;
+	assert(downstream != NULL);
+
+	void (*func)(struct xrt_frame_sink *, struct xrt_frame *);
+
+	switch (format) {
+	case XRT_FORMAT_R8G8B8: func = convert_frame_r8g8b8; break;
+	case XRT_FORMAT_L8: func = convert_frame_l8; break;
+	default: U_LOG_E("Format '%s' not supported", u_format_str(format)); return;
 	}
 
 #ifdef USE_TABLE
@@ -701,7 +860,7 @@ u_sink_create_format_converter(struct xrt_frame_context *xfctx,
 #endif
 
 	struct u_sink_converter *s = U_TYPED_CALLOC(struct u_sink_converter);
-	s->base.push_frame = receive_frame_r8g8b8;
+	s->base.push_frame = func;
 	s->node.break_apart = break_apart;
 	s->node.destroy = destroy;
 	s->downstream = downstream;
@@ -716,8 +875,32 @@ u_sink_create_to_r8g8b8_or_l8(struct xrt_frame_context *xfctx,
                               struct xrt_frame_sink *downstream,
                               struct xrt_frame_sink **out_xfs)
 {
+	assert(downstream != NULL);
+
 	struct u_sink_converter *s = U_TYPED_CALLOC(struct u_sink_converter);
-	s->base.push_frame = receive_frame_r8g8b8_or_l8;
+	s->base.push_frame = convert_frame_r8g8b8_or_l8;
+	s->node.break_apart = break_apart;
+	s->node.destroy = destroy;
+	s->downstream = downstream;
+
+#ifdef USE_TABLE
+	generate_lookup_YUV_to_RGBX();
+#endif
+
+	xrt_frame_context_add(xfctx, &s->node);
+
+	*out_xfs = &s->base;
+}
+
+void
+u_sink_create_to_r8g8b8_r8g8b8a8_r8g8b8x8_or_l8(struct xrt_frame_context *xfctx,
+                                                struct xrt_frame_sink *downstream,
+                                                struct xrt_frame_sink **out_xfs)
+{
+	assert(downstream != NULL);
+
+	struct u_sink_converter *s = U_TYPED_CALLOC(struct u_sink_converter);
+	s->base.push_frame = convert_frame_r8g8b8_r8g8b8a8_r8g8b8x8_or_l8;
 	s->node.break_apart = break_apart;
 	s->node.destroy = destroy;
 	s->downstream = downstream;
@@ -736,8 +919,10 @@ u_sink_create_to_r8g8b8_bayer_or_l8(struct xrt_frame_context *xfctx,
                                     struct xrt_frame_sink *downstream,
                                     struct xrt_frame_sink **out_xfs)
 {
+	assert(downstream != NULL);
+
 	struct u_sink_converter *s = U_TYPED_CALLOC(struct u_sink_converter);
-	s->base.push_frame = receive_frame_r8g8b8_bayer_or_l8;
+	s->base.push_frame = convert_frame_r8g8b8_bayer_or_l8;
 	s->node.break_apart = break_apart;
 	s->node.destroy = destroy;
 	s->downstream = downstream;
@@ -747,14 +932,33 @@ u_sink_create_to_r8g8b8_bayer_or_l8(struct xrt_frame_context *xfctx,
 	*out_xfs = &s->base;
 }
 
+void
+u_sink_create_to_rgb_yuv_yuyv_uyvy_or_l8(struct xrt_frame_context *xfctx,
+                                         struct xrt_frame_sink *downstream,
+                                         struct xrt_frame_sink **out_xfs)
+{
+	assert(downstream != NULL);
+
+	struct u_sink_converter *s = U_TYPED_CALLOC(struct u_sink_converter);
+	s->base.push_frame = convert_frame_rgb_yuv_yuyv_uyvy_or_l8;
+	s->node.break_apart = break_apart;
+	s->node.destroy = destroy;
+	s->downstream = downstream;
+
+	xrt_frame_context_add(xfctx, &s->node);
+
+	*out_xfs = &s->base;
+}
 
 void
 u_sink_create_to_yuv_yuyv_uyvy_or_l8(struct xrt_frame_context *xfctx,
                                      struct xrt_frame_sink *downstream,
                                      struct xrt_frame_sink **out_xfs)
 {
+	assert(downstream != NULL);
+
 	struct u_sink_converter *s = U_TYPED_CALLOC(struct u_sink_converter);
-	s->base.push_frame = receive_frame_yuv_yuyv_uyvy_or_l8;
+	s->base.push_frame = convert_frame_yuv_yuyv_uyvy_or_l8;
 	s->node.break_apart = break_apart;
 	s->node.destroy = destroy;
 	s->downstream = downstream;
@@ -769,8 +973,10 @@ u_sink_create_to_yuv_or_yuyv(struct xrt_frame_context *xfctx,
                              struct xrt_frame_sink *downstream,
                              struct xrt_frame_sink **out_xfs)
 {
+	assert(downstream != NULL);
+
 	struct u_sink_converter *s = U_TYPED_CALLOC(struct u_sink_converter);
-	s->base.push_frame = receive_frame_yuv_or_yuyv;
+	s->base.push_frame = convert_frame_yuv_or_yuyv;
 	s->node.break_apart = break_apart;
 	s->node.destroy = destroy;
 	s->downstream = downstream;
