@@ -33,11 +33,6 @@ struct android_device
 {
 	struct xrt_device base;
 	struct os_thread_helper oth;
-
-	ASensorManager *sensor_manager;
-	const ASensor *accelerometer;
-	const ASensor *gyroscope;
-	ASensorEventQueue *event_queue;
 	struct u_cardboard_distortion cardboard;
 
 
@@ -48,12 +43,12 @@ struct android_device
 		struct m_imu_3dof fusion;
 	};
 
-	enum u_logging_level ll;
+	enum u_logging_level log_level;
 };
 
 
 struct android_device *
-android_device_create();
+android_device_create(void);
 
 
 /*
@@ -62,11 +57,11 @@ android_device_create();
  *
  */
 
-#define ANDROID_TRACE(d, ...) U_LOG_XDEV_IFL_T(&d->base, d->ll, __VA_ARGS__)
-#define ANDROID_DEBUG(d, ...) U_LOG_XDEV_IFL_D(&d->base, d->ll, __VA_ARGS__)
-#define ANDROID_INFO(d, ...) U_LOG_XDEV_IFL_I(&d->base, d->ll, __VA_ARGS__)
-#define ANDROID_WARN(d, ...) U_LOG_XDEV_IFL_W(&d->base, d->ll, __VA_ARGS__)
-#define ANDROID_ERROR(d, ...) U_LOG_XDEV_IFL_E(&d->base, d->ll, __VA_ARGS__)
+#define ANDROID_TRACE(d, ...) U_LOG_XDEV_IFL_T(&d->base, d->log_level, __VA_ARGS__)
+#define ANDROID_DEBUG(d, ...) U_LOG_XDEV_IFL_D(&d->base, d->log_level, __VA_ARGS__)
+#define ANDROID_INFO(d, ...) U_LOG_XDEV_IFL_I(&d->base, d->log_level, __VA_ARGS__)
+#define ANDROID_WARN(d, ...) U_LOG_XDEV_IFL_W(&d->base, d->log_level, __VA_ARGS__)
+#define ANDROID_ERROR(d, ...) U_LOG_XDEV_IFL_E(&d->base, d->log_level, __VA_ARGS__)
 
 #ifdef __cplusplus
 }
