@@ -4,7 +4,7 @@
  * @file
  * @brief  Interface to direct OSVR HDK driver code.
  * @author Jakob Bornecrantz <jakob@collabora.com>
- * @author Ryan Pavlik <ryan.pavlik@collabora.com>
+ * @author Rylie Pavlik <rylie.pavlik@collabora.com>
  * @ingroup drv_hdk
  */
 
@@ -35,8 +35,9 @@ struct hdk_device
 	enum HDK_VARIANT variant;
 
 	struct os_thread_helper imu_thread;
+	struct os_mutex lock;
 
-	enum u_logging_level ll;
+	enum u_logging_level log_level;
 	bool disconnect_notified;
 
 	struct xrt_quat quat;
@@ -60,11 +61,11 @@ hdk_device_create(struct os_hid_device *dev, enum HDK_VARIANT variant);
  *
  */
 
-#define HDK_TRACE(d, ...) U_LOG_XDEV_IFL_T(&d->base, d->ll, __VA_ARGS__)
-#define HDK_DEBUG(d, ...) U_LOG_XDEV_IFL_D(&d->base, d->ll, __VA_ARGS__)
-#define HDK_INFO(d, ...) U_LOG_XDEV_IFL_I(&d->base, d->ll, __VA_ARGS__)
-#define HDK_WARN(d, ...) U_LOG_XDEV_IFL_W(&d->base, d->ll, __VA_ARGS__)
-#define HDK_ERROR(d, ...) U_LOG_XDEV_IFL_E(&d->base, d->ll, __VA_ARGS__)
+#define HDK_TRACE(d, ...) U_LOG_XDEV_IFL_T(&d->base, d->log_level, __VA_ARGS__)
+#define HDK_DEBUG(d, ...) U_LOG_XDEV_IFL_D(&d->base, d->log_level, __VA_ARGS__)
+#define HDK_INFO(d, ...) U_LOG_XDEV_IFL_I(&d->base, d->log_level, __VA_ARGS__)
+#define HDK_WARN(d, ...) U_LOG_XDEV_IFL_W(&d->base, d->log_level, __VA_ARGS__)
+#define HDK_ERROR(d, ...) U_LOG_XDEV_IFL_E(&d->base, d->log_level, __VA_ARGS__)
 
 #ifdef __cplusplus
 }
