@@ -4,11 +4,11 @@
 /*!
  * @file
  * @brief  Razer Hydra prober and driver code
- * @author Ryan Pavlik <ryan.pavlik@collabora.com>
+ * @author Rylie Pavlik <rylie.pavlik@collabora.com>
  * @author Jakob Bornecrantz <jakob@collabora.com>
  *
  * Portions based on the VRPN Razer Hydra driver,
- * originally written by Ryan Pavlik and available under the BSL-1.0.
+ * originally written by Rylie Pavlik and available under the BSL-1.0.
  */
 
 
@@ -39,11 +39,11 @@
  *
  */
 
-#define HYDRA_TRACE(d, ...) U_LOG_XDEV_IFL_T(&d->base, d->sys->ll, __VA_ARGS__)
-#define HYDRA_DEBUG(d, ...) U_LOG_XDEV_IFL_D(&d->base, d->sys->ll, __VA_ARGS__)
-#define HYDRA_INFO(d, ...) U_LOG_XDEV_IFL_I(&d->base, d->sys->ll, __VA_ARGS__)
-#define HYDRA_WARN(d, ...) U_LOG_XDEV_IFL_W(&d->base, d->sys->ll, __VA_ARGS__)
-#define HYDRA_ERROR(d, ...) U_LOG_XDEV_IFL_E(&d->base, d->sys->ll, __VA_ARGS__)
+#define HYDRA_TRACE(d, ...) U_LOG_XDEV_IFL_T(&d->base, d->sys->log_level, __VA_ARGS__)
+#define HYDRA_DEBUG(d, ...) U_LOG_XDEV_IFL_D(&d->base, d->sys->log_level, __VA_ARGS__)
+#define HYDRA_INFO(d, ...) U_LOG_XDEV_IFL_I(&d->base, d->sys->log_level, __VA_ARGS__)
+#define HYDRA_WARN(d, ...) U_LOG_XDEV_IFL_W(&d->base, d->sys->log_level, __VA_ARGS__)
+#define HYDRA_ERROR(d, ...) U_LOG_XDEV_IFL_E(&d->base, d->sys->log_level, __VA_ARGS__)
 
 DEBUG_GET_ONCE_LOG_OPTION(hydra_log, "HYDRA_LOG", U_LOGGING_WARN)
 
@@ -157,7 +157,7 @@ struct hydra_system
 	bool was_in_gamepad_mode;
 	int motion_attempt_number;
 
-	enum u_logging_level ll;
+	enum u_logging_level log_level;
 };
 
 /*!
@@ -328,7 +328,7 @@ hydra_system_read_data_hid(struct hydra_system *hs, timepoint_ns now)
 			return got_message ? 1 : 0;
 		}
 		if (ret != 52) {
-			U_LOG_IFL_E(hs->ll, "Unexpected data report of size %d", ret);
+			U_LOG_IFL_E(hs->log_level, "Unexpected data report of size %d", ret);
 			return -1;
 		}
 		got_message = true;
@@ -348,7 +348,7 @@ hydra_system_read_data_hid(struct hydra_system *hs, timepoint_ns now)
 		}
 
 		hs->report_time = now;
-		U_LOG_IFL_T(hs->ll,
+		U_LOG_IFL_T(hs->log_level,
 		            "\n\t"
 		            "missed: %s\n\t"
 		            "seq_no: %x\n",
@@ -369,14 +369,14 @@ hydra_system_enter_motion_control(struct hydra_system *hs, timepoint_ns now)
 
 	hs->was_in_gamepad_mode = true;
 	hs->motion_attempt_number++;
-	U_LOG_IFL_D(hs->ll,
+	U_LOG_IFL_D(hs->log_level,
 	            "Setting feature report to start motion-controller mode, "
 	            "attempt %d",
 	            hs->motion_attempt_number);
 
 	os_hid_set_feature(hs->command_hid, HYDRA_REPORT_START_MOTION, sizeof(HYDRA_REPORT_START_MOTION));
 
-	// Doing a dummy get-feature now.
+	// Doing a throwaway get-feature now.
 	uint8_t buf[91] = {0};
 	os_hid_get_feature(hs->command_hid, 0, buf, sizeof(buf));
 
@@ -443,7 +443,7 @@ hydra_device_update_input_click(struct hydra_device *hd, timepoint_ns now, int i
  *
  */
 
-static void
+static xrt_result_t
 hydra_device_update_inputs(struct xrt_device *xdev)
 {
 	struct hydra_device *hd = hydra_device(xdev);
@@ -478,12 +478,14 @@ hydra_device_update_inputs(struct xrt_device *xdev)
 		// inputs[HYDRA_INDEX_POSE].timestamp = now;
 		// inputs[HYDRA_INDEX_POSE].value.
 	}
+
+	return XRT_SUCCESS;
 }
 
-static void
+static xrt_result_t
 hydra_device_get_tracked_pose(struct xrt_device *xdev,
                               enum xrt_input_name name,
-                              uint64_t at_timestamp_ns,
+                              int64_t at_timestamp_ns,
                               struct xrt_space_relation *out_relation)
 {
 	struct hydra_device *hd = hydra_device(xdev);
@@ -503,6 +505,8 @@ hydra_device_get_tracked_pose(struct xrt_device *xdev,
 	// HYDRA_SPEW(hd, "GET_TRACKED_POSE (%f, %f, %f) (%f, %f, %f, %f) ",
 	// pos.x,
 	//            pos.y, pos.z, quat.x, quat.y, quat.z, quat.w);
+
+	return XRT_SUCCESS;
 }
 
 static void
@@ -526,7 +530,7 @@ hydra_system_remove_child(struct hydra_system *hs, struct hydra_device *hd)
 		if (hs->data_hid != NULL && hs->command_hid != NULL && hs->sm.current_state == HYDRA_SM_REPORTING &&
 		    hs->was_in_gamepad_mode) {
 
-			U_LOG_IFL_D(hs->ll,
+			U_LOG_IFL_D(hs->log_level,
 			            "hydra: Sending command to re-enter gamepad mode "
 			            "and pausing while it takes effect.");
 
@@ -570,7 +574,7 @@ hydra_device_destroy(struct xrt_device *xdev)
 int
 hydra_found(struct xrt_prober *xp,
             struct xrt_prober_device **devices,
-            size_t num_devices,
+            size_t device_count,
             size_t index,
             cJSON *attached_data,
             struct xrt_device **out_xdevs)
@@ -594,10 +598,10 @@ hydra_found(struct xrt_prober *xp,
 	struct hydra_system *hs = U_TYPED_CALLOC(struct hydra_system);
 	hs->base.type = XRT_TRACKING_TYPE_HYDRA;
 	snprintf(hs->base.name, XRT_TRACKING_NAME_LEN, "%s", "Razer Hydra magnetic tracking");
-	// Dummy transform from local space to base.
-	hs->base.offset.position.y = 1.0f;
-	hs->base.offset.position.z = -0.25f;
-	hs->base.offset.orientation.w = 1.0f;
+	// Arbitrary static transform from local space to base.
+	hs->base.initial_offset.position.y = 1.0f;
+	hs->base.initial_offset.position.z = -0.25f;
+	hs->base.initial_offset.orientation.w = 1.0f;
 
 	hs->data_hid = data_hid;
 	hs->command_hid = command_hid;
@@ -613,7 +617,7 @@ hydra_found(struct xrt_prober *xp,
 	hs->report_counter = -1;
 	hs->refs = 2;
 
-	hs->ll = debug_get_log_option_hydra_log();
+	hs->log_level = debug_get_log_option_hydra_log();
 
 	// Populate the individual devices
 	for (size_t i = 0; i < 2; ++i) {
@@ -625,6 +629,7 @@ hydra_found(struct xrt_prober *xp,
 		// hs->base.set_output = hydra_device_set_output;
 		hd->base.name = XRT_DEVICE_HYDRA;
 		snprintf(hd->base.str, XRT_DEVICE_NAME_LEN, "%s %i", "Razer Hydra Controller", (int)(i + 1));
+		snprintf(hd->base.serial, XRT_DEVICE_NAME_LEN, "%s %i", "Razer Hydra Controller", (int)(i + 1));
 		SET_INPUT(1_CLICK);
 		SET_INPUT(2_CLICK);
 		SET_INPUT(3_CLICK);
