@@ -17,7 +17,11 @@
  */
 
 #if defined(__ANDROID__)
+#include "xrt/xrt_config_android.h"
 #define XRT_OS_ANDROID
+#if defined(XRT_FEATURE_AHARDWARE_BUFFER)
+#define XRT_OS_ANDROID_USE_AHB
+#endif
 #define XRT_OS_LINUX
 #define XRT_OS_UNIX
 #define XRT_OS_WAS_AUTODETECTED
@@ -34,6 +38,9 @@
 #define XRT_OS_WAS_AUTODETECTED
 #endif
 
+#if defined(__MINGW32__)
+#define XRT_ENV_MINGW
+#endif
 
 #ifndef XRT_OS_WAS_AUTODETECTED
 #error "OS type not found during compile"
