@@ -1,9 +1,11 @@
-// Copyright 2019-2020, Collabora, Ltd.
+// Copyright 2019-2023, Collabora, Ltd.
+// Copyright 2023, NVIDIA CORPORATION.
 // SPDX-License-Identifier: BSL-1.0
 /*!
  * @file
  * @brief  Action related API entrypoint functions.
  * @author Jakob Bornecrantz <jakob@collabora.com>
+ * @author Korcan Hussein <korcan.hussein@collabora.com>
  * @ingroup oxr_api
  */
 
@@ -12,54 +14,177 @@
 #include "oxr_handle.h"
 
 #include "util/u_debug.h"
+#include "util/u_trace_marker.h"
 
 #include "oxr_api_funcs.h"
 #include "oxr_api_verify.h"
+#include "oxr_chain.h"
 #include "oxr_subaction.h"
 
 #include <stdio.h>
-#include <inttypes.h>
 
 #include "bindings/b_generated_bindings.h"
 
+
+/*
+ *
+ * Dpad functions.
+ *
+ */
+
+#ifdef XR_EXT_dpad_binding
+static XrResult
+process_dpad(struct oxr_logger *log,
+             struct oxr_instance *inst,
+             struct oxr_dpad_state *state,
+             const XrInteractionProfileDpadBindingEXT *dpad,
+             path_verify_fn_t dpad_emulator_fn,
+             const char *prefix,
+             const char *ip_str)
+{
+	const char *str = NULL;
+	size_t length = 0;
+	XrResult ret;
+
+	ret = oxr_path_get_string(log, inst, dpad->binding, &str, &length);
+	if (ret != XR_SUCCESS) {
+		return oxr_error(log, XR_ERROR_PATH_INVALID, "(%s->binding == %" PRIu64 ") is not a valid path", prefix,
+		                 dpad->binding);
+	}
+
+	if (!dpad_emulator_fn(&inst->extensions, inst->openxr_version.major_minor, str, length)) {
+		return oxr_error(log, XR_ERROR_PATH_UNSUPPORTED,
+		                 "(%s->binding == \"%s\") is not a valid dpad binding path for profile \"%s\"", prefix,
+		                 str, ip_str);
+	}
+
+	ret = oxr_verify_XrInteractionProfileDpadBindingEXT(log, dpad, prefix);
+	if (ret != XR_SUCCESS) {
+		return ret;
+	}
+
+	if (dpad->actionSet == XR_NULL_HANDLE) {
+		return oxr_error(log, XR_ERROR_HANDLE_INVALID, "(%s->actionSet == XR_NULL_HANDLE)", prefix);
+	}
+
+	struct oxr_action_set *act_set = XRT_CAST_OXR_HANDLE_TO_PTR(struct oxr_action_set *, dpad->actionSet);
+	if (act_set->handle.debug != OXR_XR_DEBUG_ACTIONSET) {
+		return oxr_error(log, XR_ERROR_HANDLE_INVALID, "(%s->actionSet == %p)", prefix, (void *)act_set);
+	}
+
+	struct oxr_dpad_entry *entry = oxr_dpad_state_get_or_add(state, act_set->act_set_key);
+	if (entry->key == 0) {
+		entry->key = act_set->act_set_key;
+		assert(act_set->act_set_key != 0);
+	}
+
+	bool added = false;
+	for (size_t i = 0; i < ARRAY_SIZE(entry->dpads); i++) {
+		// Have we found a empty slot, add it.
+		if (entry->dpads[i].binding == XR_NULL_PATH) {
+			struct oxr_dpad_binding_modification dpad_binding = {
+			    .binding = dpad->binding,
+			    .settings = {
+			        .forceThreshold = dpad->forceThreshold,
+			        .forceThresholdReleased = dpad->forceThresholdReleased,
+			        .centerRegion = dpad->centerRegion,
+			        .wedgeAngle = dpad->wedgeAngle,
+			        .isSticky = dpad->isSticky,
+			    }};
+			entry->dpads[i] = dpad_binding;
+
+			entry->dpad_count++;
+			added = true;
+			break;
+		}
+
+		if (entry->dpads[i].binding == dpad->binding) {
+			return oxr_error(
+			    log, XR_ERROR_VALIDATION_FAILURE,
+			    "(%s->[actionSet == \"%s\", binding == \"%s\"]) pair is already added to profile \"%s\"",
+			    prefix, act_set->data->name, str, ip_str);
+		}
+	}
+
+	if (!added) {
+		return oxr_error(log, XR_ERROR_RUNTIME_FAILURE, "Failed to add dpad binding!");
+	}
+
+	return XR_SUCCESS;
+}
+#endif
+
+
 /*
  *
  * Session - action functions.
  *
  */
 
-XrResult
+XRAPI_ATTR XrResult XRAPI_CALL
 oxr_xrSyncActions(XrSession session, const XrActionsSyncInfo *syncInfo)
 {
+	OXR_TRACE_MARKER();
+
 	struct oxr_session *sess;
 	struct oxr_logger log;
 	OXR_VERIFY_SESSION_AND_INIT_LOG(&log, session, sess, "xrSyncActions");
+	OXR_VERIFY_SESSION_NOT_LOST(&log, sess);
 	OXR_VERIFY_ARG_TYPE_AND_NOT_NULL(&log, syncInfo, XR_TYPE_ACTIONS_SYNC_INFO);
 
+	const XrActiveActionSetPrioritiesEXT *active_priorities = NULL;
+
+#ifdef OXR_HAVE_EXT_active_action_set_priority
+	active_priorities = OXR_GET_OUTPUT_FROM_CHAIN(syncInfo->next, XR_TYPE_ACTIVE_ACTION_SET_PRIORITIES_EXT,
+	                                              const XrActiveActionSetPrioritiesEXT);
+
+	// ensure that if the active priority override is present,
+	// the EXT_active_action_set_priority extension is enabled
+	if (active_priorities) {
+		OXR_VERIFY_EXTENSION(&log, sess->sys->inst, EXT_active_action_set_priority);
+	}
+#endif
+
+	struct xrt_system_roles sys_roles = XRT_STRUCT_INIT;
+	xrt_system_devices_get_roles(sess->sys->xsysd, &sys_roles);
+	{
+		os_mutex_lock(&sess->sys->sync_actions_mutex);
+		if (sess->sys->dynamic_roles_cache.generation_id < sys_roles.generation_id) {
+			sess->sys->dynamic_roles_cache = sys_roles;
+			oxr_session_update_action_bindings(&log, sess);
+		}
+		os_mutex_unlock(&sess->sys->sync_actions_mutex);
+	}
+
 	if (syncInfo->countActiveActionSets == 0) {
-		return oxr_error(&log, XR_ERROR_VALIDATION_FAILURE, "(syncInfo->countActiveActionSets == 0)");
+		// nothing to do
+		return XR_SUCCESS;
 	}
 
 	for (uint32_t i = 0; i < syncInfo->countActiveActionSets; i++) {
 		struct oxr_action_set *act_set = NULL;
 		OXR_VERIFY_ACTIONSET_NOT_NULL(&log, syncInfo->activeActionSets[i].actionSet, act_set);
 
-		XrResult res = oxr_verify_subaction_path_sync(&log, sess->sys->inst,
+		XrResult res = oxr_verify_subaction_path_sync(&log, sess->sys->inst, act_set,
 		                                              syncInfo->activeActionSets[i].subactionPath, i);
 		if (res != XR_SUCCESS) {
 			return res;
 		}
 	}
 
-	return oxr_action_sync_data(&log, sess, syncInfo->countActiveActionSets, syncInfo->activeActionSets);
+	return oxr_action_sync_data(&log, sess, syncInfo->countActiveActionSets, syncInfo->activeActionSets,
+	                            active_priorities);
 }
 
-XrResult
+XRAPI_ATTR XrResult XRAPI_CALL
 oxr_xrAttachSessionActionSets(XrSession session, const XrSessionActionSetsAttachInfo *bindInfo)
 {
+	OXR_TRACE_MARKER();
+
 	struct oxr_session *sess;
 	struct oxr_logger log;
 	OXR_VERIFY_SESSION_AND_INIT_LOG(&log, session, sess, "xrAttachSessionActionSets");
+	OXR_VERIFY_SESSION_NOT_LOST(&log, sess);
 	OXR_VERIFY_ARG_TYPE_AND_NOT_NULL(&log, bindInfo, XR_TYPE_SESSION_ACTION_SETS_ATTACH_INFO);
 
 	if (sess->act_set_attachments != NULL) {
@@ -82,10 +207,12 @@ oxr_xrAttachSessionActionSets(XrSession session, const XrSessionActionSetsAttach
 	return oxr_session_attach_action_sets(&log, sess, bindInfo);
 }
 
-XrResult
+XRAPI_ATTR XrResult XRAPI_CALL
 oxr_xrSuggestInteractionProfileBindings(XrInstance instance,
                                         const XrInteractionProfileSuggestedBinding *suggestedBindings)
 {
+	OXR_TRACE_MARKER();
+
 	struct oxr_instance *inst;
 	struct oxr_logger log;
 	OXR_VERIFY_INSTANCE_AND_INIT_LOG(&log, instance, inst, "xrSuggestInteractionProfileBindings");
@@ -94,48 +221,62 @@ oxr_xrSuggestInteractionProfileBindings(XrInstance instance,
 	if (suggestedBindings->countSuggestedBindings == 0) {
 		return oxr_error(&log, XR_ERROR_VALIDATION_FAILURE,
 		                 "(suggestedBindings->countSuggestedBindings "
-		                 "== 0) can not suggest 0 bindings");
+		                 "== 0) cannot suggest 0 bindings");
 	}
 
 	XrPath ip = suggestedBindings->interactionProfile;
-	const char *str = NULL;
-	size_t length;
+	const char *ip_str = NULL;
+	size_t ip_length = 0;
 
-	XrResult ret = oxr_path_get_string(&log, inst, ip, &str, &length);
+	XrResult ret = oxr_path_get_string(&log, inst, ip, &ip_str, &ip_length);
 	if (ret != XR_SUCCESS) {
 		oxr_error(&log, ret, "(suggestedBindings->countSuggestedBindings == 0x%08" PRIx64 ") invalid path", ip);
 	}
 
 	// Used in the loop that verifies the suggested bindings paths.
-	bool (*func)(const char *, size_t) = NULL;
+	path_verify_fn_t subpath_fn = NULL;
+	path_verify_fn_t dpad_path_fn = NULL;
+	path_verify_fn_t dpad_emulator_fn = NULL;
+	ext_verify_fn_t ext_verify_fn = NULL;
+	bool has_dpad = inst->extensions.EXT_dpad_binding;
 
-	if (ip == inst->path_cache.khr_simple_controller) {
-		func = oxr_verify_khr_simple_controller_subpath;
-	} else if (ip == inst->path_cache.google_daydream_controller) {
-		func = oxr_verify_google_daydream_controller_subpath;
-	} else if (ip == inst->path_cache.htc_vive_controller) {
-		func = oxr_verify_htc_vive_controller_subpath;
-	} else if (ip == inst->path_cache.htc_vive_pro) {
-		func = oxr_verify_htc_vive_pro_subpath;
-	} else if (ip == inst->path_cache.microsoft_motion_controller) {
-		func = oxr_verify_microsoft_motion_controller_subpath;
-	} else if (ip == inst->path_cache.microsoft_xbox_controller) {
-		func = oxr_verify_microsoft_xbox_controller_subpath;
-	} else if (ip == inst->path_cache.oculus_go_controller) {
-		func = oxr_verify_oculus_go_controller_subpath;
-	} else if (ip == inst->path_cache.oculus_touch_controller) {
-		func = oxr_verify_oculus_touch_controller_subpath;
-	} else if (ip == inst->path_cache.valve_index_controller) {
-		func = oxr_verify_valve_index_controller_subpath;
-	} else if (ip == inst->path_cache.mndx_ball_on_a_stick_controller) {
-		func = oxr_verify_mndx_ball_on_a_stick_controller_subpath;
-	} else {
+	struct profile_template *interaction_profile_template = NULL;
+	for (uint32_t i = 0; i < ARRAY_SIZE(profile_templates); i++) {
+		if (ip == profile_templates[i].path_cache) {
+			subpath_fn = profile_templates[i].subpath_fn;
+			dpad_path_fn = profile_templates[i].dpad_path_fn;
+			dpad_emulator_fn = profile_templates[i].dpad_emulator_fn;
+			ext_verify_fn = profile_templates[i].ext_verify_fn;
+			interaction_profile_template = &profile_templates[i];
+			break;
+		}
+	}
+
+	if (interaction_profile_template == NULL) {
 		return oxr_error(&log, XR_ERROR_PATH_UNSUPPORTED,
 		                 "(suggestedBindings->interactionProfile == \"%s\") is not "
 		                 "a supported interaction profile",
-		                 str);
+		                 ip_str);
 	}
 
+	bool ext_supported;
+	bool ext_enabled;
+	ext_verify_fn(&inst->extensions, inst->openxr_version.major_minor, &ext_supported, &ext_enabled);
+	if (!ext_supported) {
+		return oxr_error(
+		    &log, XR_ERROR_PATH_UNSUPPORTED,
+		    "(suggestedBindings->interactionProfile == \"%s\") used but XR_%s not supported by runtime", ip_str,
+		    interaction_profile_template->extension_name);
+	}
+	if (!ext_enabled) {
+		return oxr_error(&log, XR_ERROR_PATH_UNSUPPORTED,
+		                 "(suggestedBindings->interactionProfile == \"%s\") used but XR_%s not enabled", ip_str,
+		                 interaction_profile_template->extension_name);
+	}
+
+	// Needed in various paths here.
+	const char *str = NULL;
+	size_t length;
 
 	for (size_t i = 0; i < suggestedBindings->countSuggestedBindings; i++) {
 		const XrActionSuggestedBinding *s = &suggestedBindings->suggestedBindings[i];
@@ -158,26 +299,88 @@ oxr_xrSuggestInteractionProfileBindings(XrInstance instance,
 			                 i, s->binding);
 		}
 
-		if (!func(str, length)) {
-			return oxr_error(&log, XR_ERROR_PATH_UNSUPPORTED,
-			                 "(suggestedBindings->suggestedBindings[%zu]->"
-			                 "binding == \"%s\") is not a valid path",
-			                 i, str);
+		if (subpath_fn(&inst->extensions, inst->openxr_version.major_minor, str, length)) {
+			continue;
 		}
+
+#ifdef XR_EXT_dpad_binding
+		if (dpad_path_fn(&inst->extensions, inst->openxr_version.major_minor, str, length)) {
+			if (!has_dpad) {
+				return oxr_error(
+				    &log, XR_ERROR_PATH_UNSUPPORTED,
+				    "(suggestedBindings->suggestedBindings[%zu]->binding == \"%s\") is is a dpad path, "
+				    "but XR_EXT_dpad_binding is not enabled, for profile \"%s\"",
+				    i, str, ip_str);
+			}
+			continue;
+		}
+#endif
+
+		return oxr_error(&log, XR_ERROR_PATH_UNSUPPORTED,
+		                 "(suggestedBindings->suggestedBindings[%zu]->binding == \"%s\") is not a valid "
+		                 "binding path for profile \"%s\"",
+		                 i, str, ip_str);
 	}
 
-	return oxr_action_suggest_interaction_profile_bindings(&log, inst, suggestedBindings);
+
+	/*
+	 * Binding modifications.
+	 */
+
+	const XrBindingModificationsKHR *mods = OXR_GET_INPUT_FROM_CHAIN(
+	    suggestedBindings->next, XR_TYPE_BINDING_MODIFICATIONS_KHR, XrBindingModificationsKHR);
+
+	struct oxr_dpad_state dpad_state = {0};
+#ifdef XR_EXT_dpad_binding
+	if (has_dpad) {
+		if (!oxr_dpad_state_init(&dpad_state)) {
+			return oxr_error(&log, XR_ERROR_RUNTIME_FAILURE, " Failed to init dpad state!");
+		}
+
+		char temp[512] = {0};
+		for (uint32_t i = 0; mods != NULL && i < mods->bindingModificationCount; i++) {
+			const XrBindingModificationBaseHeaderKHR *mod = mods->bindingModifications[i];
+			const XrInteractionProfileDpadBindingEXT *dpad = OXR_GET_INPUT_FROM_CHAIN(
+			    mod, XR_TYPE_INTERACTION_PROFILE_DPAD_BINDING_EXT, XrInteractionProfileDpadBindingEXT);
+			if (dpad == NULL) {
+				continue;
+			}
+
+			snprintf(temp, sizeof(temp),
+			         "suggestedBindings->next<XrBindingModificationsKHR>->bindingModifications[%u]->next<"
+			         "XrInteractionProfileDpadBindingEXT>",
+			         i);
+
+			ret = process_dpad(&log, inst, &dpad_state, dpad, dpad_emulator_fn, temp, ip_str);
+			if (ret != XR_SUCCESS) {
+				// Teardown the state.
+				oxr_dpad_state_deinit(&dpad_state);
+				return ret;
+			}
+		}
+	}
+#endif
+
+
+	/*
+	 * Everything verified.
+	 */
+
+	return oxr_action_suggest_interaction_profile_bindings(&log, inst, suggestedBindings, &dpad_state);
 }
 
-XrResult
+XRAPI_ATTR XrResult XRAPI_CALL
 oxr_xrGetCurrentInteractionProfile(XrSession session,
                                    XrPath topLevelUserPath,
                                    XrInteractionProfileState *interactionProfile)
 {
+	OXR_TRACE_MARKER();
+
 	struct oxr_instance *inst = NULL;
 	struct oxr_session *sess = NULL;
 	struct oxr_logger log;
 	OXR_VERIFY_SESSION_AND_INIT_LOG(&log, session, sess, "xrGetCurrentInteractionProfile");
+	OXR_VERIFY_SESSION_NOT_LOST(&log, sess);
 	OXR_VERIFY_ARG_TYPE_AND_NOT_NULL(&log, interactionProfile, XR_TYPE_INTERACTION_PROFILE_STATE);
 
 	// Short hand.
@@ -190,7 +393,7 @@ oxr_xrGetCurrentInteractionProfile(XrSession session,
 	}
 
 	if (!oxr_path_is_valid(&log, inst, topLevelUserPath)) {
-		return oxr_error(&log, XR_ERROR_PATH_INVALID, "(topLevelUserPath == %zu) Is not a valid path",
+		return oxr_error(&log, XR_ERROR_PATH_INVALID, "(topLevelUserPath == %" PRId64 ") Is not a valid path",
 		                 topLevelUserPath);
 	}
 
@@ -217,17 +420,20 @@ oxr_xrGetCurrentInteractionProfile(XrSession session,
 	return oxr_action_get_current_interaction_profile(&log, sess, topLevelUserPath, interactionProfile);
 }
 
-XrResult
+XRAPI_ATTR XrResult XRAPI_CALL
 oxr_xrGetInputSourceLocalizedName(XrSession session,
                                   const XrInputSourceLocalizedNameGetInfo *getInfo,
                                   uint32_t bufferCapacityInput,
                                   uint32_t *bufferCountOutput,
                                   char *buffer)
 {
+	OXR_TRACE_MARKER();
+
 	struct oxr_instance *inst = NULL;
 	struct oxr_session *sess;
 	struct oxr_logger log;
 	OXR_VERIFY_SESSION_AND_INIT_LOG(&log, session, sess, "xrGetInputSourceLocalizedName");
+	OXR_VERIFY_SESSION_NOT_LOST(&log, sess);
 	OXR_VERIFY_ARG_TYPE_AND_NOT_NULL(&log, getInfo, XR_TYPE_INPUT_SOURCE_LOCALIZED_NAME_GET_INFO);
 
 	// Short hand.
@@ -245,8 +451,8 @@ oxr_xrGetInputSourceLocalizedName(XrSession session,
 	}
 
 	if (!oxr_path_is_valid(&log, inst, getInfo->sourcePath)) {
-		return oxr_error(&log, XR_ERROR_PATH_INVALID, "(getInfo->sourcePath == %zu) Is not a valid path",
-		                 getInfo->sourcePath);
+		return oxr_error(&log, XR_ERROR_PATH_INVALID,
+		                 "(getInfo->sourcePath == %" PRId64 ") Is not a valid path", getInfo->sourcePath);
 	}
 
 	const XrInputSourceLocalizedNameFlags all = XR_INPUT_SOURCE_LOCALIZED_NAME_USER_PATH_BIT |
@@ -255,11 +461,12 @@ oxr_xrGetInputSourceLocalizedName(XrSession session,
 
 	if ((getInfo->whichComponents & ~all) != 0) {
 		return oxr_error(&log, XR_ERROR_VALIDATION_FAILURE,
-		                 "(getInfo->whichComponents == %08zx) contains invalid bits", getInfo->whichComponents);
+		                 "(getInfo->whichComponents == %08" PRIx64 ") contains invalid bits",
+		                 getInfo->whichComponents);
 	}
 
 	if (getInfo->whichComponents == 0) {
-		return oxr_error(&log, XR_ERROR_VALIDATION_FAILURE, "(getInfo->whichComponents == 0) can not be zero");
+		return oxr_error(&log, XR_ERROR_VALIDATION_FAILURE, "(getInfo->whichComponents == 0) cannot be zero");
 	}
 
 	return oxr_action_get_input_source_localized_name(&log, sess, getInfo, bufferCapacityInput, bufferCountOutput,
@@ -273,9 +480,11 @@ oxr_xrGetInputSourceLocalizedName(XrSession session,
  *
  */
 
-XrResult
+XRAPI_ATTR XrResult XRAPI_CALL
 oxr_xrCreateActionSet(XrInstance instance, const XrActionSetCreateInfo *createInfo, XrActionSet *actionSet)
 {
+	OXR_TRACE_MARKER();
+
 	struct oxr_action_set *act_set = NULL;
 	struct oxr_instance *inst = NULL;
 	struct u_hashset_item *d = NULL;
@@ -322,9 +531,11 @@ oxr_xrCreateActionSet(XrInstance instance, const XrActionSetCreateInfo *createIn
 	return XR_SUCCESS;
 }
 
-XrResult
+XRAPI_ATTR XrResult XRAPI_CALL
 oxr_xrDestroyActionSet(XrActionSet actionSet)
 {
+	OXR_TRACE_MARKER();
+
 	struct oxr_action_set *act_set;
 	struct oxr_logger log;
 	OXR_VERIFY_ACTIONSET_AND_INIT_LOG(&log, actionSet, act_set, "xrDestroyActionSet");
@@ -339,9 +550,11 @@ oxr_xrDestroyActionSet(XrActionSet actionSet)
  *
  */
 
-XrResult
+XRAPI_ATTR XrResult XRAPI_CALL
 oxr_xrCreateAction(XrActionSet actionSet, const XrActionCreateInfo *createInfo, XrAction *action)
 {
+	OXR_TRACE_MARKER();
+
 	struct oxr_action_set *act_set;
 	struct u_hashset_item *d = NULL;
 	struct oxr_action *act = NULL;
@@ -402,9 +615,11 @@ oxr_xrCreateAction(XrActionSet actionSet, const XrActionCreateInfo *createInfo,
 	return XR_SUCCESS;
 }
 
-XrResult
+XRAPI_ATTR XrResult XRAPI_CALL
 oxr_xrDestroyAction(XrAction action)
 {
+	OXR_TRACE_MARKER();
+
 	struct oxr_action *act;
 	struct oxr_logger log;
 	OXR_VERIFY_ACTION_AND_INIT_LOG(&log, action, act, "xrDestroyAction");
@@ -412,15 +627,18 @@ oxr_xrDestroyAction(XrAction action)
 	return oxr_handle_destroy(&log, &act->handle);
 }
 
-XrResult
+XRAPI_ATTR XrResult XRAPI_CALL
 oxr_xrGetActionStateBoolean(XrSession session, const XrActionStateGetInfo *getInfo, XrActionStateBoolean *data)
 {
+	OXR_TRACE_MARKER();
+
 	struct oxr_session *sess = NULL;
 	struct oxr_action *act = NULL;
 	struct oxr_subaction_paths subaction_paths = {0};
 	struct oxr_logger log;
 	XrResult ret;
 	OXR_VERIFY_SESSION_AND_INIT_LOG(&log, session, sess, "xrGetActionStateBoolean");
+	OXR_VERIFY_SESSION_NOT_LOST(&log, sess);
 	OXR_VERIFY_ARG_TYPE_AND_NOT_NULL(&log, data, XR_TYPE_ACTION_STATE_BOOLEAN);
 	OXR_VERIFY_ARG_TYPE_AND_NOT_NULL(&log, getInfo, XR_TYPE_ACTION_STATE_GET_INFO);
 	OXR_VERIFY_ACTION_NOT_NULL(&log, getInfo->action, act);
@@ -438,15 +656,18 @@ oxr_xrGetActionStateBoolean(XrSession session, const XrActionStateGetInfo *getIn
 	return oxr_action_get_boolean(&log, sess, act->act_key, subaction_paths, data);
 }
 
-XrResult
+XRAPI_ATTR XrResult XRAPI_CALL
 oxr_xrGetActionStateFloat(XrSession session, const XrActionStateGetInfo *getInfo, XrActionStateFloat *data)
 {
+	OXR_TRACE_MARKER();
+
 	struct oxr_session *sess = NULL;
 	struct oxr_action *act = NULL;
 	struct oxr_subaction_paths subaction_paths = {0};
 	struct oxr_logger log;
 	XrResult ret;
 	OXR_VERIFY_SESSION_AND_INIT_LOG(&log, session, sess, "xrGetActionStateFloat");
+	OXR_VERIFY_SESSION_NOT_LOST(&log, sess);
 	OXR_VERIFY_ARG_TYPE_AND_NOT_NULL(&log, data, XR_TYPE_ACTION_STATE_FLOAT);
 	OXR_VERIFY_ARG_TYPE_AND_NOT_NULL(&log, getInfo, XR_TYPE_ACTION_STATE_GET_INFO);
 	OXR_VERIFY_ACTION_NOT_NULL(&log, getInfo->action, act);
@@ -464,15 +685,18 @@ oxr_xrGetActionStateFloat(XrSession session, const XrActionStateGetInfo *getInfo
 	return oxr_action_get_vector1f(&log, sess, act->act_key, subaction_paths, data);
 }
 
-XrResult
+XRAPI_ATTR XrResult XRAPI_CALL
 oxr_xrGetActionStateVector2f(XrSession session, const XrActionStateGetInfo *getInfo, XrActionStateVector2f *data)
 {
+	OXR_TRACE_MARKER();
+
 	struct oxr_session *sess = NULL;
 	struct oxr_action *act = NULL;
 	struct oxr_subaction_paths subaction_paths = {0};
 	struct oxr_logger log;
 	XrResult ret;
 	OXR_VERIFY_SESSION_AND_INIT_LOG(&log, session, sess, "xrGetActionStateVector2f");
+	OXR_VERIFY_SESSION_NOT_LOST(&log, sess);
 	OXR_VERIFY_ARG_TYPE_AND_NOT_NULL(&log, data, XR_TYPE_ACTION_STATE_VECTOR2F);
 	OXR_VERIFY_ARG_TYPE_AND_NOT_NULL(&log, getInfo, XR_TYPE_ACTION_STATE_GET_INFO);
 	OXR_VERIFY_ACTION_NOT_NULL(&log, getInfo->action, act);
@@ -490,15 +714,18 @@ oxr_xrGetActionStateVector2f(XrSession session, const XrActionStateGetInfo *getI
 	return oxr_action_get_vector2f(&log, sess, act->act_key, subaction_paths, data);
 }
 
-XrResult
+XRAPI_ATTR XrResult XRAPI_CALL
 oxr_xrGetActionStatePose(XrSession session, const XrActionStateGetInfo *getInfo, XrActionStatePose *data)
 {
+	OXR_TRACE_MARKER();
+
 	struct oxr_session *sess = NULL;
 	struct oxr_action *act = NULL;
 	struct oxr_subaction_paths subaction_paths = {0};
 	struct oxr_logger log;
 	XrResult ret;
 	OXR_VERIFY_SESSION_AND_INIT_LOG(&log, session, sess, "xrGetActionStatePose");
+	OXR_VERIFY_SESSION_NOT_LOST(&log, sess);
 	OXR_VERIFY_ARG_TYPE_AND_NOT_NULL(&log, data, XR_TYPE_ACTION_STATE_POSE);
 	OXR_VERIFY_ARG_TYPE_AND_NOT_NULL(&log, getInfo, XR_TYPE_ACTION_STATE_GET_INFO);
 	OXR_VERIFY_ACTION_NOT_NULL(&log, getInfo->action, act);
@@ -516,17 +743,20 @@ oxr_xrGetActionStatePose(XrSession session, const XrActionStateGetInfo *getInfo,
 	return oxr_action_get_pose(&log, sess, act->act_key, subaction_paths, data);
 }
 
-XrResult
+XRAPI_ATTR XrResult XRAPI_CALL
 oxr_xrEnumerateBoundSourcesForAction(XrSession session,
                                      const XrBoundSourcesForActionEnumerateInfo *enumerateInfo,
                                      uint32_t sourceCapacityInput,
                                      uint32_t *sourceCountOutput,
                                      XrPath *sources)
 {
+	OXR_TRACE_MARKER();
+
 	struct oxr_session *sess = NULL;
 	struct oxr_action *act = NULL;
 	struct oxr_logger log;
 	OXR_VERIFY_SESSION_AND_INIT_LOG(&log, session, sess, "xrEnumerateBoundSourcesForAction");
+	OXR_VERIFY_SESSION_NOT_LOST(&log, sess);
 	OXR_VERIFY_ARG_TYPE_AND_NOT_NULL(&log, enumerateInfo, XR_TYPE_BOUND_SOURCES_FOR_ACTION_ENUMERATE_INFO);
 	OXR_VERIFY_ACTION_NOT_NULL(&log, enumerateInfo->action, act);
 
@@ -547,17 +777,20 @@ oxr_xrEnumerateBoundSourcesForAction(XrSession session,
  *
  */
 
-XrResult
+XRAPI_ATTR XrResult XRAPI_CALL
 oxr_xrApplyHapticFeedback(XrSession session,
                           const XrHapticActionInfo *hapticActionInfo,
                           const XrHapticBaseHeader *hapticEvent)
 {
+	OXR_TRACE_MARKER();
+
 	struct oxr_session *sess = NULL;
 	struct oxr_action *act = NULL;
 	struct oxr_subaction_paths subaction_paths = {0};
 	struct oxr_logger log;
 	XrResult ret;
 	OXR_VERIFY_SESSION_AND_INIT_LOG(&log, session, sess, "xrApplyHapticFeedback");
+	OXR_VERIFY_SESSION_NOT_LOST(&log, sess);
 	OXR_VERIFY_ARG_TYPE_AND_NOT_NULL(&log, hapticActionInfo, XR_TYPE_HAPTIC_ACTION_INFO);
 	OXR_VERIFY_ARG_TYPE_AND_NOT_NULL(&log, hapticEvent, XR_TYPE_HAPTIC_VIBRATION);
 	OXR_VERIFY_ACTION_NOT_NULL(&log, hapticActionInfo->action, act);
@@ -575,15 +808,18 @@ oxr_xrApplyHapticFeedback(XrSession session,
 	return oxr_action_apply_haptic_feedback(&log, sess, act->act_key, subaction_paths, hapticEvent);
 }
 
-XrResult
+XRAPI_ATTR XrResult XRAPI_CALL
 oxr_xrStopHapticFeedback(XrSession session, const XrHapticActionInfo *hapticActionInfo)
 {
+	OXR_TRACE_MARKER();
+
 	struct oxr_session *sess = NULL;
 	struct oxr_action *act = NULL;
 	struct oxr_subaction_paths subaction_paths = {0};
 	struct oxr_logger log;
 	XrResult ret;
 	OXR_VERIFY_SESSION_AND_INIT_LOG(&log, session, sess, "xrStopHapticFeedback");
+	OXR_VERIFY_SESSION_NOT_LOST(&log, sess);
 	OXR_VERIFY_ARG_TYPE_AND_NOT_NULL(&log, hapticActionInfo, XR_TYPE_HAPTIC_ACTION_INFO);
 	OXR_VERIFY_ACTION_NOT_NULL(&log, hapticActionInfo->action, act);
 
