@@ -1,10 +1,11 @@
-// Copyright 2020, Collabora, Ltd.
+// Copyright 2020-2024, Collabora, Ltd.
 // SPDX-License-Identifier: BSL-1.0
 /*!
  * @file
  * @brief  Shared default implementation of the instance: pieces that are used
  * whether or not there's a compositor.
  * @author Jakob Bornecrantz <jakob@collabora.com>
+ * @author Rylie Pavlik <rylie.pavlik@collabora.com>
  */
 #pragma once
 
@@ -12,9 +13,17 @@
 
 #include "xrt/xrt_prober.h"
 #include "xrt/xrt_instance.h"
+#include "xrt/xrt_config_os.h"
+#ifdef XRT_OS_ANDROID
+#include "xrt/xrt_android.h"
+#endif // XRT_OS_ANDROID
 
 #include "util/u_misc.h"
+#include "util/u_trace_marker.h"
 
+#ifdef XRT_OS_ANDROID
+#include "android/android_instance_base.h"
+#endif
 
 /*
  *
@@ -25,7 +34,7 @@
 /*!
  * Main "real" instance implementation.
  *
- * Used in instances both with and without compositor access.
+ * Used in instances both with and without compositor usage.
  *
  * @implements xrt_instance
  */
@@ -33,6 +42,9 @@ struct t_instance
 {
 	struct xrt_instance base;
 	struct xrt_prober *xp;
+#ifdef XRT_OS_ANDROID
+	struct android_instance_base android;
+#endif
 };
 
 static inline struct t_instance *
@@ -48,34 +60,34 @@ t_instance(struct xrt_instance *xinst)
  *
  */
 
-static int
-t_instance_select(struct xrt_instance *xinst, struct xrt_device **xdevs, size_t num_xdevs)
-{
-	struct t_instance *tinst = t_instance(xinst);
-
-	int ret = xrt_prober_probe(tinst->xp);
-	if (ret < 0) {
-		return ret;
-	}
-
-	return xrt_prober_select(tinst->xp, xdevs, num_xdevs);
-}
-
-static int
+static xrt_result_t
 t_instance_get_prober(struct xrt_instance *xinst, struct xrt_prober **out_xp)
 {
+	XRT_TRACE_MARKER();
+
 	struct t_instance *tinst = t_instance(xinst);
 
+	if (tinst->xp == NULL) {
+		return XRT_ERROR_PROBER_NOT_SUPPORTED;
+	}
+
 	*out_xp = tinst->xp;
 
-	return 0;
+	return XRT_SUCCESS;
 }
 
 static void
 t_instance_destroy(struct xrt_instance *xinst)
 {
+	XRT_TRACE_MARKER();
+
 	struct t_instance *tinst = t_instance(xinst);
 
 	xrt_prober_destroy(&tinst->xp);
+
+#ifdef XRT_OS_ANDROID
+	android_instance_base_cleanup(&tinst->android, xinst);
+#endif // XRT_OS_ANDROID
+
 	free(tinst);
 }
