@@ -9,7 +9,10 @@
 
 #include "util/u_var.h"
 #include "gui_sdl2.h"
+#include "util/u_trace_marker.h"
+#include <string.h>
 
+U_TRACE_TARGET_SETUP(U_TRACE_WHICH_SERVICE)
 
 int
 main(int argc, char **argv)
@@ -20,6 +23,8 @@ main(int argc, char **argv)
 	// Need to do this as early as possible.
 	u_var_force_on();
 
+	u_trace_marker_init();
+
 	ret = gui_sdl2_init(&p);
 	if (ret != 0) {
 		gui_sdl2_quit(&p);
@@ -35,12 +40,15 @@ main(int argc, char **argv)
 	// First scene to start with.
 	if (argc >= 2 && strcmp("debug", argv[1]) == 0) {
 		// We have created a prober select devices now.
-		gui_prober_select(&p.base);
 		gui_scene_debug(&p.base);
 	} else if (argc >= 2 && strcmp("calibrate", argv[1]) == 0) {
 		gui_scene_select_video_calibrate(&p.base);
+	} else if (argc >= 2 && strcmp("tracking_overrides", argv[1]) == 0) {
+		gui_scene_tracking_overrides(&p.base);
+	} else if (argc >= 2 && strcmp("record", argv[1]) == 0) {
+		gui_scene_record(&p.base, argc >= 3 ? argv[2] : NULL);
 	} else if (argc >= 2 && strcmp("remote", argv[1]) == 0) {
-		gui_scene_remote(&p.base);
+		gui_scene_remote(&p.base, argc >= 3 ? argv[2] : NULL);
 	} else {
 		gui_scene_main_menu(&p.base);
 	}
